function [C,W] = linear_nystroem(coord,indices)

%function K = linear(coord)
%
% Computes an linear kernel matrix from the input coordinates
%
%INPUTS
% coord =  a matrix containing all samples as rows
%
%OUTPUTS
% K = the linear kernel matrix <.,.>
%
%
% For more info, see www.kernel-methods.net

%
%Author: Frank-Michael Schleif
% modified version to allow nystroem appoximation
if(length(indices) > 1000)
    error('You indices selection is very large - consider a smaller set - stop now');
    return;
end
n=size(coord,1);
iNumberOfLandmarks = length(indices);
W = zeros(iNumberOfLandmarks);
C = zeros(n,iNumberOfLandmarks);
coord_sub = coord(indices,:);
K=coord_sub*coord_sub';                         %thant K(ind,ind)
W=pinv(K);
C(indices,:) = K;

% fill in rest for K(:,ind);         % C
vIndicesRest        = setdiff([1:n],indices);
iNumberOfRestPoints = length(vIndicesRest);
coord_sub_rest      = coord(vIndicesRest,:);
for(k=1:iNumberOfRestPoints)        
    for(l=1:iNumberOfLandmarks)
          C(vIndicesRest(k),l) = coord_sub_rest(k,:) * coord_sub(l,:)';
    end
    if(mod(k,100)==0)
        warning('Long running op: %f percent\n',k/iNumberOfRestPoints);
    end
end 
