function [C,W] = rbf_nystroem(coord,sig,indices)

%function K = rbf(coord,sig)
%
% Computes an rbf kernel matrix from the input coordinates
%
%INPUTS
% coord =  a matrix containing all samples as rows
% sig = sigma, the kernel width; squared distances are divided by
%       squared sig in the exponent
%
%OUTPUTS
% K = the rbf kernel matrix ( = exp(-1/(2*sigma^2)*(coord*coord')^2) )
%
%
% For more info, see www.kernel-methods.net

%
%Author: Tijl De Bie, february 2003. Adapted: october 2004 (for speedup).
% modified version to allow nystroem appoximation
if(length(indices) > 1000)
    error('You indices selection is very large - consider a smaller set - stop now');
    return;
end
n=size(coord,1);
iNumberOfLandmarks = length(indices);
W = zeros(iNumberOfLandmarks);
C = zeros(n,iNumberOfLandmarks);
coord_sub = coord(indices,:);
K=coord_sub*coord_sub'/sig^2;
d=diag(K);
K=K-ones(iNumberOfLandmarks,1)*d'/2;
K=K-d*ones(1,iNumberOfLandmarks)/2;
K=exp(K);
W=pinv(K);
C(indices,:) = K;

% rest
vIndicesRest        = setdiff([1:n],indices);
iNumberOfRestPoints = length(vIndicesRest);
coord_sub_rest      = coord(vIndicesRest,:);
if(iNumberOfRestPoints < 1000)
    KL    = coord_sub_rest*coord_sub_rest';
    vSelf = diag(KL);
    for(k=1:iNumberOfRestPoints)
        for(l=1:iNumberOfLandmarks)
            C(vIndicesRest(k),l) = exp(-1/(2*sig^2) * (vSelf(k) - 2.0 * coord_sub_rest(k,:) * coord_sub(l,:)' + sig^2*d(l)));
        end
    end
else    
    for(k=1:iNumberOfRestPoints)        
        for(l=1:iNumberOfLandmarks)
            C(vIndicesRest(k),l) = exp(-1/(2*sig^2) * (coord_sub_rest(k,:)*coord_sub_rest(k,:)' - 2.0 * coord_sub_rest(k,:) * coord_sub(l,:)' + sig^2*d(l)));
        end
        if(mod(k,100)==0)
            warning('Long running op: %f percent\n',k/iNumberOfRestPoints);
        end
    end 
end
